@@ -862,7 +862,7 @@ sub parse_hdrfields {
 }
 
 sub parse_reqhdr {
-    my ($data,$hdr) = @_;
+    my ($data,$hdr,$external_length) = @_;
     $data =~m{\A
 	([A-Z]{2,20})[\040\t]+          # $1: method
 	(\S+)[\040\t]+                  # $2: path/URI
@@ -916,13 +916,15 @@ sub parse_reqhdr {
     }
 
     if ( $METHODS_WITHOUT_RQBODY{$method} ) {
-	# Ignore any kind of length information.
-	$hdr->{content_length} = 0;
-	delete $hdr->{chunked};
+	# Complain if the client announced a body.
+	return "no body allowed with $method"
+	    if $hdr->{content_length} or $hdr->{chunked};
 
     } elsif ( $METHODS_WITH_RQBODY{$method} ) {
 	return "content-length or transfer-encoding chunked must be given with method $method"
-	    if ! $hdr->{chunked} and ! defined $hdr->{content_length};
+	    if ! $hdr->{chunked}
+	    and ! defined $hdr->{content_length}
+	    and ! $external_length;
 
     } elsif ( ! $hdr->{chunked} ) {
 	# if not given content-length is considered 0
@@ -1206,11 +1208,18 @@ sub upgrade_websocket {
 		    $clen = undef;
 		}
 
+		if (defined $clen) {
+		    $data_frame->{bytes_left} = [$clenhi,$clen];
+		} else {
+		    delete $data_frame->{bytes_left};
+		}
 		$obj->in_wsdata($dir,$data,$eom,$time,$data_frame);
-		delete $data_frame->{init};
 		if ($eom) {
 		    $data_frame = $current_frame = undef;
 		    $self->{gap_upto}[$dir] = 0;
+		} else {
+		    delete $data_frame->{init};
+		    delete $data_frame->{header};
 		}
 		return;
 	    }
@@ -1254,11 +1263,17 @@ sub upgrade_websocket {
 		    $clen = undef;
 		}
 		if ($data_frame && $current_frame == $data_frame) {
+		    if (defined $clen) {
+			$data_frame->{bytes_left} = [$clenhi,$clen];
+		    } else {
+			delete $data_frame->{bytes_left};
+		    }
 		    $obj->in_wsdata($dir,$fwd,$eom,$time,$data_frame);
 		    if ($eom) {
 			$data_frame = undef;
 		    } else {
 			delete $data_frame->{init};
+			delete $data_frame->{header};
 			$current_frame->{mask_offset}
 			    = (($current_frame->{mask_offset}||0) + length($fwd)) % 4
 			    if defined $clen;
@@ -1371,7 +1386,7 @@ sub upgrade_websocket {
 	    }
 
 	    # done with frame header
-	    substr($rbuf,0,$off,'');
+	    $current_frame->{header} = substr($rbuf,0,$off,'');
 	    goto PARSE_DATA;
 
 	    # Done
@@ -1614,8 +1629,9 @@ This will be called after a Websocket upgrade when receiving a control frame.
 C<$dir> is 0 for data from client, 1 for data from server.
 C<$data> is the unmasked payload of the frame.
 C<$frameinfo> is a blessed hash reference which contains the C<opcode> of the
-frame and the C<mask> (binary). For a close frame it will also contain the
-extracted C<status> code and the C<reason>.
+frame, the C<mask> (binary) and C<header> for the frame header.
+For a close frame it will also contain the extracted C<status> code and the
+C<reason>.
 
 To get the unmasked payload call C<< $frameinfo->unmask($masked_data) >>.
 
@@ -1635,7 +1651,19 @@ is done and the FIN bit was set on the frame.
 C<$frameinfo> is a blessed hash reference which contains the data type as
 C<opcode>. This will be the original opcode of the starting frame in case of
 fragmented transfer. It will also contain the C<mask> (binary) of the current
-frame.
+frame. 
+
+If this is the initial part of the data (i.e. initial frame in possibly
+fragmented data and initial data inside this frame) it will also have C<init>
+set to true inside C<$frameinfo>.
+
+If there are still unread data within the frame C<$frameinfo> will contain
+C<bytes_left> as C<<[hi,low]>> where C<hi> and C<low> are the upper and lower
+32 bit parts of the number of outstanding bytes.
+
+If this call to C<in_wsdata> was caused by the start of a new frame and not
+further data in the same frame C<header> will be set to the header of this new
+frame. In all other cases C<header> is not set.
 
 To get the unmasked payload call C<< $frameinfo->unmask($masked_data) >>.
 
@@ -1741,12 +1769,15 @@ It will return any remaining data in C<$header> which could not be interpreted
 as proper C<key:value> pairs. If the message contains no errors it will thus
 return C<''>.
 
-=item parse_reqhdr($string,\%header) -> $bad_header
+=item parse_reqhdr($string,\%header,[$external_length]) -> $bad_header
 
 This will parse the given C<$string> as a request header and extract information
 into \%header. These information then later will be given to
 C<in_request_header>. See there for more details about the contents of the hash.
 
+If C<$external_length> is true it will not complain if a content-length is
+required but not defined.
+
 =item parse_rsphdr($string,\%request,\%header) -> $bad_header
 
 This will parse the given C<$string> as a response header and extract
